==> wmpasman-0.8.4.1.tar.gz: wmpasman-0.8.4.1/wmpasman/password.c <==
#include "../config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>

#include "wmpasman.h"
#include "globals.h"
#include "crypto.h"
#include "die.h"
#include "password.h"

#ifndef S_IRWX
#define S_IRWX (S_IRWXU|S_IRWXG|S_IRWXO)
#endif
#define S_SAFE (S_IRWX&(S_IRUSR|S_IWUSR))


struct _Password {
    char line1[LINE_CHARS+1];
    char line2[LINE_CHARS+1];
    int passlen;
    char *enc_pass;
};


Password password_new(const char *line1, const char *line2, char *pass){
    Password p;
    int len, i, flag;
    Crypto c;
    char *IV;
    char buf[11];

    len=strlen(pass);
    if(len>=MAX_PASSWORD_LEN){
        set_error("Password too long!");
        return NULL;
    }

    if((p=malloc(sizeof(*p)))==NULL) return NULL;
    if((p->enc_pass=malloc(len))==NULL) return NULL;
    flag=0;
    for(i=0; i<LINE_CHARS; i++){
        if(line1[i]=='\0') flag|=1;
        if(line2[i]=='\0') flag|=2;
        p->line1[i]=(flag&1)?'\0':line1[i];
        p->line2[i]=(flag&2)?'\0':line2[i];
    }
    p->line1[LINE_CHARS]='\0';
    p->line2[LINE_CHARS]='\0';
    p->passlen=len;
    snprintf(buf, 11, "%x", p->passlen);

    if((IV=crypto_new_IV(NULL, p->line1, p->line2, buf, IV_END))==NULL){
        password_free(p);
        return NULL;
    }
    c=crypto_new(IV);
    free(IV);
    if(c==NULL){ password_free(p); return NULL; }
    i=crypto_encrypt_binary(c, len, pass);
    crypto_free(c);
    if(i){ password_free(p); return NULL; }
    memcpy(p->enc_pass, pass, len);
    return p;
}

void password_free(Password p){
    free(p->enc_pass);
    free(p);
}

void password_get_labels(Password p, const char **line1, const char **line2){
    *line1=p->line1;
    *line2=p->line2;
}

const char *password_get_pass(Password p){
    int i;
    Crypto c;
    char *IV;
    char buf[11];

    if(p->passlen>=MAX_PASSWORD_LEN){
        set_error("Password too long!");
        return NULL;
    }
    memcpy(password_buffer, p->enc_pass, p->passlen);

    snprintf(buf, 11, "%x", p->passlen);
    if((IV=crypto_new_IV(NULL, p->line1, p->line2, buf, IV_END))==NULL)
        return NULL;
    c=crypto_new(IV);
    free(IV);
    if(c==NULL) return NULL;
    i=crypto_decrypt(c, p->passlen, password_buffer);
    crypto_free(c);
    if(i) return NULL;
    password_buffer[p->passlen]='\0';
    return password_buffer;
}

int password_recrypt_pass(Password p){
    char buf[11];
    const char *IV;

    if(p->passlen>=MAX_PASSWORD_LEN){
        set_error("Password too long!");
        return -1;
    }

    snprintf(buf, 11, "%x", p->passlen);
    IV=crypto_recrypt_salted(p->passlen, p->enc_pass, NULL, p->line1, p->line2, buf, IV_END);
    if(IV){
        free((void *)IV);
        return 0;
    }
    return 1;
}

/************************************************************/

/* Verify appropriate permissions on the password file */
static ssize_t verify_file(const char *fname){
    struct stat filestat;

    close(open(fname, O_CREAT|O_RDONLY, S_SAFE));

    if(lstat(fname, &filestat)==-1){
        debug(DEBUG_INFO, "Couldn't stat %s", fname);
        return -1;
    }

    if(!S_ISREG(filestat.st_mode)){
        errno=0;
        debug(DEBUG_INFO, "%s is not a regular file", fname);
        set_error("%s is not a regular file", fname);
        return -1;
    }

    if((filestat.st_mode & S_IRWX) != S_SAFE){
        debug(DEBUG_INFO, "Bad permissions on %s, are %03o, should be %03o", fname, filestat.st_mode & S_IRWX, S_SAFE);
        set_error("Bad permissions on %s, are %03o, should be %03o", fname, filestat.st_mode & S_IRWX, S_SAFE);
        return -1;
    }

    return filestat.st_size;
}

int pwlist_load(PWList list, const char *filename){
    int fp;
    char *IV;
    int IV_len;
    Crypto c;
    ssize_t fsize;
    int i, numpw, len;
    char l1[LINE_CHARS+1], l2[LINE_CHARS+1];
    unsigned char gnum;
    Password p;

    fsize=verify_file(filename);
    if(fsize<0) return -1;
    if(fsize==0) return 0;

    if((IV=crypto_new_IV(&IV_len, IV_EMPTY))==NULL) return -1;
    if((fp=open(filename, O_RDONLY))<0){
        free(IV);
        return -1;
    }
    if(IV_len!=0){
        errno=0;
        if(read(fp, IV, IV_len)!=IV_len){
            free(IV);
            close(fp);
            if(!errno) set_error("Unexpected EOF while reading IV");
            return -1;
        }
        fsize-=IV_len;
    }

    c=crypto_new(IV);
    free(IV);
    if(c==NULL){
        close(fp);
        return -1;
    }

#define READ(buf, len) { \
        errno=0; \
        if(read(fp, buf, len)!=len){ \
            if(!errno) ERR(); \
            close(fp); \
            crypto_free(c); \
            return -1; \
        } \
        fsize-=len; \
        if(crypto_decrypt(c, len, buf)){ \
            close(fp); \
            crypto_free(c); \
            return -1; \
        } \
    }
#define READINT(var) { \
        char buf[2]; \
        READ(buf, 2); \
        var=buf[0]+(buf[1]<<8); \
    }
#define ERR() set_error("Unexpected EOF while reading file header")
#define DECRYPT_FAILED() { \
        set_error("Decryption failed (wrong passphrase?)"); \
        close(fp); \
        crypto_free(c); \
        return -1; \
    }

    /* Skip 0-255 garbage bytes */
    READ(&gnum, sizeof(gnum));
    i=gnum;
    if(i>=MAX_PASSWORD_LEN || i+sizeof(numpw)>fsize) DECRYPT_FAILED();
    READ(password_buffer, i);

    /* Read in the number of passwords */
    READINT(numpw);
    if(numpw<0 || numpw*sizeof(len)*3>fsize) DECRYPT_FAILED();

    password_buffer[MAX_PASSWORD_LEN-1]='\0';
    for(i=0; i<numpw; i++){
#undef ERR
#define ERR() set_error("Unexpected EOF while reading password #%d", i)
        READINT(len);
        if(len<0 || len>LINE_CHARS || len+sizeof(len)*2+(numpw-i-1)*sizeof(len)*3>fsize) DECRYPT_FAILED();
        READ(l1, len);
        l1[len]='\0';
        READINT(len);
        if(len<0 || len>LINE_CHARS || len+sizeof(len)+(numpw-i-1)*sizeof(len)*3>fsize) DECRYPT_FAILED();
        READ(l2, len);
        l2[len]='\0';
        READINT(len);
        if(len<0 || len>=MAX_PASSWORD_LEN || len+(numpw-i-1)*sizeof(len)*3>fsize) DECRYPT_FAILED();
        READ(password_buffer, len);
        password_buffer[len]='\0';
        if((p=password_new(l1, l2, password_buffer))==NULL){
            close(fp);
            crypto_free(c);
            return -1;
        }
        if(pwlist_push(list, p)){
            close(fp);
            crypto_free(c);
            return -1;
        }
    }

    errno=0;
    if(read(fp, password_buffer, 1)!=0){
        debug(DEBUG_INFO, "Garbage at end of %s", filename);
    }
    crypto_free(c);
    close(fp);
    return 0;
#undef ERR
#undef READ
#undef READINT
#undef DECRYPT_FAILED
}

int pwlist_save(PWList list, const char *filename){
    int fp;
    char *IV;
    int IV_len;
    Crypto c;
    int i, len;
    unsigned char gnum;

    if(verify_file(filename)<0) return -1;

    if((IV=crypto_new_IV(&IV_len, IV_END))==NULL) return -1;
    if((fp=open(filename, O_WRONLY | O_TRUNC))<0){
        free(IV);
        return -1;
    }
    if(IV_len!=0){
        if(write(fp, IV, IV_len)!=IV_len){
            free(IV);
            close(fp);
            return -1;
        }
    }

    c=crypto_new(IV);
    free(IV);
    if(c==NULL){
        close(fp);
        return -1;
    }

#define WRITE(buf, len) { \
        if(crypto_encrypt_binary(c, len, buf) || write(fp, buf, len)!=len){ \
            close(fp); \
            crypto_free(c); \
            return -1; \
        } \
    }
#define WRITEINT(var) { \
        char buf[2]; \
        buf[0]=var&0xff; \
        buf[1]=(var>>8)&0xff; \
        WRITE(buf, 2); \
    }

    /* Write 0-255 garbage bytes */
    do {
        if(randomize(&gnum, sizeof(gnum))){
            close(fp);
            crypto_free(c);
            return -1;
        }
        len=gnum;
    } while(len>=MAX_PASSWORD_LEN);
    WRITE(&gnum, sizeof(gnum));
    if(randomize(password_buffer, len)){
        close(fp);
        crypto_free(c);
        return -1;
    }
    WRITE(password_buffer, len);
    
    /* Write out the number of passwords */
    WRITEINT(list->num);

    /* Write out the passwords */
    for(i=0; i<list->num; i++){
        strcpy(password_buffer, list->p[i]->line1);
        len=strlen(password_buffer);
        WRITEINT(len);
        WRITE(password_buffer, len);
        strcpy(password_buffer, list->p[i]->line2);
        len=strlen(password_buffer);
        WRITEINT(len);
        WRITE(password_buffer, len);
        IV=(char *)password_get_pass(list->p[i]);
        WRITEINT(list->p[i]->passlen);
        WRITE(IV, list->p[i]->passlen);
    }

    crypto_free(c);
    close(fp);
    return 0;
#undef WRITE
#undef WRITEINT
}

PWList pwlist_new(int capacity){
    PWList p;
    int i;
    
    if((p=malloc(sizeof(*p)))==NULL) return NULL;
    p->num=0;
    p->capacity=0;
    p->p=NULL;
    if(capacity>0 && (p->p=malloc(sizeof(*(p->p))*capacity))!=NULL){
        p->capacity=capacity;
        for(i=0; i<p->capacity; i++){
            p->p[i]=NULL;
        }
    }
    return p;
}

void pwlist_free(PWList p){
    if(p->p) free(p->p);
    free(p);
}

void pwlist_free_all(PWList p){
    int i;
    for(i=0; i<p->num; i++){
        if(p->p[i]) password_free(p->p[i]);
    }
    pwlist_free(p);
}

int pwlist_count(PWList p){
    return p->num;
}

int pwlist_push(PWList p, Password pw){
    Password *list;
    int i;

    if(p->num == p->capacity){
        if(p->capacity<=0) p->capacity=1;
        if((list=realloc(p->p, sizeof(*(p->p))*2*p->capacity))==NULL) return -1;
        p->p=list;
        for(i=p->capacity; i<2*p->capacity; i++){
            p->p[i]=NULL;
        }
        p->capacity*=2;
    }
    p->p[p->num++]=pw;
    return 0;
}

int pwlist_find_i(PWList p, const char *line1, const char *line2){
    int i;

    for(i=0; i<p->num; i++){
        if(!strcmp(line1, p->p[i]->line1) &&!strcmp(line2, p->p[i]->line2))
            return i;
    }
    return -1;
}

Password pwlist_find_p(PWList p, const char *line1, const char *line2){
    int i;

    for(i=0; i<p->num; i++){
        if(!strcmp(line1, p->p[i]->line1) &&!strcmp(line2, p->p[i]->line2))
            return p->p[i];
    }
    return NULL;
}

Password pwlist_get_i(PWList p, int idx){
    if(idx<0 || idx>=p->num) return NULL;
    return p->p[idx];
}

int pwlist_get_p(PWList p, Password pw){
    int i;

    for(i=0; i<p->num; i++){
        if(p->p[i]==pw) return i;
    }
    return -1;
}

Password pwlist_remove_i(PWList p, int idx){
    Password pw;

    if(idx<0 || idx>=p->num) return NULL;
    pw=p->p[idx];
    for(; idx<p->num-1; idx++){
        p->p[idx]=p->p[idx+1];
    }
    p->p[--p->num]=NULL;
    return pw;
}

int pwlist_remove_p(PWList p, Password pw){
    int i, idx=-1;

    for(i=0; i<p->num; i++){
        if(p->p[i]!=pw) continue;
        idx=i;
        break;
    }
    if(i>=p->num) return -1;
    for(; i<p->num-1; i++){
        p->p[i]=p->p[i+1];
    }
    p->p[--p->num]=NULL;
    return idx;
}

int pwlist_free_i(PWList p, int idx){
    Password pw;

    pw=pwlist_remove_i(p, idx);
    if(pw){
        password_free(pw);
        return 0;
    }
    return -1;
}

int pwlist_free_p(PWList p, Password pw){
    if(pwlist_remove_p(p, pw)>=0){
        password_free(pw);
        return 0;
    }
    return -1;
}

int pwlist_recrypt_passes(PWList p){
    int i;
    
    for(i=0; i<p->num; i++){
        if(password_recrypt_pass(p->p[i])) return -1;
    }
    return 0;
}

==> wmpasman-0.8.5.3.tar.gz: wmpasman-0.8.5.3/wmpasman/password.c <==
#include "../config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>

#include "wmpasman.h"
#include "globals.h"
#include "crypto.h"
#include "die.h"
#include "password.h"

#ifndef S_IRWX
#define S_IRWX (S_IRWXU|S_IRWXG|S_IRWXO)
#endif
#define S_SAFE (S_IRWX&(S_IRUSR|S_IWUSR))


struct _Password {
    char line1[LINE_CHARS+1];
    char line2[LINE_CHARS+1];
    int passlen;
    char *enc_pass;
};


Password password_new(const char *line1, const char *line2, char *pass){
    Password p;
    int len, i, flag;
    Crypto c;
    char *IV;
    char buf[11];

    len=strlen(pass);
    if(len>=MAX_PASSWORD_LEN){
        set_error("Password too long!");
        return NULL;
    }

    if((p=malloc(sizeof(*p)))==NULL) return NULL;
    if((p->enc_pass=malloc(len))==NULL) return NULL;
    flag=0;
    for(i=0; i<LINE_CHARS; i++){
        if(line1[i]=='\0') flag|=1;
        if(line2[i]=='\0') flag|=2;
        p->line1[i]=(flag&1)?'\0':line1[i];
        p->line2[i]=(flag&2)?'\0':line2[i];
    }
    p->line1[LINE_CHARS]='\0';
    p->line2[LINE_CHARS]='\0';
    p->passlen=len;
    snprintf(buf, 11, "%x", p->passlen);

    if((IV=crypto_new_IV(NULL, p->line1, p->line2, buf, IV_END))==NULL){
        password_free(p);
        return NULL;
    }
    c=crypto_new(IV);
    free(IV);
    if(c==NULL){ password_free(p); return NULL; }
    i=crypto_encrypt_binary(c, len, pass);
    crypto_free(c);
    if(i){ password_free(p); return NULL; }
    memcpy(p->enc_pass, pass, len);
    return p;
}

void password_free(Password p){
    free(p->enc_pass);
    free(p);
}

void password_get_labels(Password p, const char **line1, const char **line2){
    *line1=p->line1;
    *line2=p->line2;
}

const char *password_get_pass(Password p){
    int i;
    Crypto c;
    char *IV;
    char buf[11];

    if(p->passlen>=MAX_PASSWORD_LEN){
        set_error("Password too long!");
        return NULL;
    }
    memcpy(password_buffer, p->enc_pass, p->passlen);

    snprintf(buf, 11, "%x", p->passlen);
    if((IV=crypto_new_IV(NULL, p->line1, p->line2, buf, IV_END))==NULL)
        return NULL;
    c=crypto_new(IV);
    free(IV);
    if(c==NULL) return NULL;
    i=crypto_decrypt(c, p->passlen, password_buffer);
    crypto_free(c);
    if(i) return NULL;
    password_buffer[p->passlen]='\0';
    return password_buffer;
}

int password_recrypt_pass(Password p){
    char buf[11];
    const char *IV;

    if(p->passlen>=MAX_PASSWORD_LEN){
        set_error("Password too long!");
        return -1;
    }

    snprintf(buf, 11, "%x", p->passlen);
    IV=crypto_recrypt_salted(p->passlen, p->enc_pass, NULL, p->line1, p->line2, buf, IV_END);
    if(IV){
        free((void *)IV);
        return 0;
    }
    return 1;
}

/************************************************************/

/* Verify appropriate permissions on the password file */
static ssize_t verify_file(const char *fname){
    struct stat filestat;

    close(open(fname, O_CREAT|O_RDONLY, S_SAFE));

    if(lstat(fname, &filestat)==-1){
        debug(DEBUG_INFO, "Couldn't stat %s", fname);
        return -1;
    }

    if(!S_ISREG(filestat.st_mode)){
        errno=0;
        debug(DEBUG_INFO, "%s is not a regular file", fname);
        set_error("%s is not a regular file", fname);
        return -1;
    }

    if((filestat.st_mode & S_IRWX) != S_SAFE){
        debug(DEBUG_INFO, "Bad permissions on %s, are %03o, should be %03o", fname, filestat.st_mode & S_IRWX, S_SAFE);
        set_error("Bad permissions on %s, are %03o, should be %03o", fname, filestat.st_mode & S_IRWX, S_SAFE);
        return -1;
    }

    return filestat.st_size;
}

int pwlist_load(PWList list, const char *filename){
    int fp;
    char *IV;
    int IV_len;
    Crypto c;
    ssize_t fsize;
    int i, numpw, len;
    char l1[LINE_CHARS+1], l2[LINE_CHARS+1];
    unsigned char gnum;
    Password p;

    fsize=verify_file(filename);
    if(fsize<0) return -1;
    if(fsize==0) return 0;

    if((IV=crypto_new_IV(&IV_len, IV_EMPTY))==NULL) return -1;
    if((fp=open(filename, O_RDONLY))<0){
        free(IV);
        return -1;
    }
    if(IV_len!=0){
        errno=0;
        if(read(fp, IV, IV_len)!=IV_len){
            free(IV);
            close(fp);
            if(!errno) set_error("Unexpected EOF while reading IV");
            return -1;
        }
        fsize-=IV_len;
    }

    c=crypto_new(IV);
    free(IV);
    if(c==NULL){
        close(fp);
        return -1;
    }

#define READ(buf, len) { \
        errno=0; \
        if(read(fp, buf, len)!=len){ \
            if(!errno) ERR(); \
            close(fp); \
            crypto_free(c); \
            return -1; \
        } \
        fsize-=len; \
        if(crypto_decrypt(c, len, buf)){ \
            close(fp); \
            crypto_free(c); \
            return -1; \
        } \
    }
#define READINT(var) { \
        char buf[2]; \
        READ(buf, 2); \
        var=buf[0]+(buf[1]<<8); \
    }
#define ERR() set_error("Unexpected EOF while reading file header")
#define DECRYPT_FAILED() { \
        set_error("Decryption failed (wrong passphrase?)"); \
        close(fp); \
        crypto_free(c); \
        return -1; \
    }

    /* Skip 0-255 garbage bytes */
    READ((char *)&gnum, sizeof(gnum));
    i=gnum;
    if(i>=MAX_PASSWORD_LEN || i+sizeof(numpw)>fsize) DECRYPT_FAILED();
    READ(password_buffer, i);

    /* Read in the number of passwords */
    READINT(numpw);
    if(numpw<0 || numpw*sizeof(len)*3>fsize) DECRYPT_FAILED();

    password_buffer[MAX_PASSWORD_LEN-1]='\0';
    for(i=0; i<numpw; i++){
#undef ERR
#define ERR() set_error("Unexpected EOF while reading password #%d", i)
        READINT(len);
        if(len<0 || len>LINE_CHARS || len+sizeof(len)*2+(numpw-i-1)*sizeof(len)*3>fsize) DECRYPT_FAILED();
        READ(l1, len);
        l1[len]='\0';
        READINT(len);
        if(len<0 || len>LINE_CHARS || len+sizeof(len)+(numpw-i-1)*sizeof(len)*3>fsize) DECRYPT_FAILED();
        READ(l2, len);
        l2[len]='\0';
        READINT(len);
        if(len<0 || len>=MAX_PASSWORD_LEN || len+(numpw-i-1)*sizeof(len)*3>fsize) DECRYPT_FAILED();
        READ(password_buffer, len);
        password_buffer[len]='\0';
        if((p=password_new(l1, l2, password_buffer))==NULL){
            close(fp);
            crypto_free(c);
            return -1;
        }
        if(pwlist_push(list, p)){
            close(fp);
            crypto_free(c);
            return -1;
        }
    }

    errno=0;
    if(read(fp, password_buffer, 1)!=0){
        debug(DEBUG_INFO, "Garbage at end of %s", filename);
    }
    crypto_free(c);
    close(fp);
    return 0;
#undef ERR
#undef READ
#undef READINT
#undef DECRYPT_FAILED
}

int pwlist_save(PWList list, const char *filename){
    int fp;
    char *IV;
    int IV_len;
    Crypto c;
    int i, len;
    unsigned char gnum;

    if(verify_file(filename)<0) return -1;

    if((IV=crypto_new_IV(&IV_len, IV_END))==NULL) return -1;
    if((fp=open(filename, O_WRONLY | O_TRUNC))<0){
        free(IV);
        return -1;
    }
    if(IV_len!=0){
        if(write(fp, IV, IV_len)!=IV_len){
            free(IV);
            close(fp);
            return -1;
        }
    }

    c=crypto_new(IV);
    free(IV);
    if(c==NULL){
        close(fp);
        return -1;
    }

#define WRITE(buf, len) { \
        if(crypto_encrypt_binary(c, len, buf) || write(fp, buf, len)!=len){ \
            close(fp); \
            crypto_free(c); \
            return -1; \
        } \
    }
#define WRITEINT(var) { \
        char buf[2]; \
        buf[0]=var&0xff; \
        buf[1]=(var>>8)&0xff; \
        WRITE(buf, 2); \
    }

    /* Write 0-255 garbage bytes */
    do {
        if(randomize((char *)&gnum, sizeof(gnum))){
            close(fp);
            crypto_free(c);
            return -1;
        }
        len=gnum;
    } while(len>=MAX_PASSWORD_LEN);
    WRITE((char *)&gnum, sizeof(gnum));
    if(randomize(password_buffer, len)){
        close(fp);
        crypto_free(c);
        return -1;
    }
    WRITE(password_buffer, len);
    
    /* Write out the number of passwords */
    WRITEINT(list->num);

    /* Write out the passwords */
    for(i=0; i<list->num; i++){
        strcpy(password_buffer, list->p[i]->line1);
        len=strlen(password_buffer);
        WRITEINT(len);
        WRITE(password_buffer, len);
        strcpy(password_buffer, list->p[i]->line2);
        len=strlen(password_buffer);
        WRITEINT(len);
        WRITE(password_buffer, len);
        IV=(char *)password_get_pass(list->p[i]);
        WRITEINT(list->p[i]->passlen);
        WRITE(IV, list->p[i]->passlen);
    }

    crypto_free(c);
    close(fp);
    return 0;
#undef WRITE
#undef WRITEINT
}

PWList pwlist_new(int capacity){
    PWList p;
    int i;
    
    if((p=malloc(sizeof(*p)))==NULL) return NULL;
    p->num=0;
    p->capacity=0;
    p->p=NULL;
    if(capacity>0 && (p->p=malloc(sizeof(*(p->p))*capacity))!=NULL){
        p->capacity=capacity;
        for(i=0; i<p->capacity; i++){
            p->p[i]=NULL;
        }
    }
    return p;
}

void pwlist_free(PWList p){
    if(p->p) free(p->p);
    free(p);
}

void pwlist_free_all(PWList p){
    int i;
    for(i=0; i<p->num; i++){
        if(p->p[i]) password_free(p->p[i]);
    }
    pwlist_free(p);
}

int pwlist_count(PWList p){
    return p->num;
}

int pwlist_push(PWList p, Password pw){
    Password *list;
    int i;

    if(p->num == p->capacity){
        if(p->capacity<=0) p->capacity=1;
        if((list=realloc(p->p, sizeof(*(p->p))*2*p->capacity))==NULL) return -1;
        p->p=list;
        for(i=p->capacity; i<2*p->capacity; i++){
            p->p[i]=NULL;
        }
        p->capacity*=2;
    }
    p->p[p->num++]=pw;
    return 0;
}

int pwlist_find_i(PWList p, const char *line1, const char *line2){
    int i;

    for(i=0; i<p->num; i++){
        if(!strcmp(line1, p->p[i]->line1) &&!strcmp(line2, p->p[i]->line2))
            return i;
    }
    return -1;
}

Password pwlist_find_p(PWList p, const char *line1, const char *line2){
    int i;

    for(i=0; i<p->num; i++){
        if(!strcmp(line1, p->p[i]->line1) &&!strcmp(line2, p->p[i]->line2))
            return p->p[i];
    }
    return NULL;
}

Password pwlist_get_i(PWList p, int idx){
    if(idx<0 || idx>=p->num) return NULL;
    return p->p[idx];
}

int pwlist_get_p(PWList p, Password pw){
    int i;

    for(i=0; i<p->num; i++){
        if(p->p[i]==pw) return i;
    }
    return -1;
}

Password pwlist_remove_i(PWList p, int idx){
    Password pw;

    if(idx<0 || idx>=p->num) return NULL;
    pw=p->p[idx];
    for(; idx<p->num-1; idx++){
        p->p[idx]=p->p[idx+1];
    }
    p->p[--p->num]=NULL;
    return pw;
}

int pwlist_remove_p(PWList p, Password pw){
    int i, idx=-1;

    for(i=0; i<p->num; i++){
        if(p->p[i]!=pw) continue;
        idx=i;
        break;
    }
    if(i>=p->num) return -1;
    for(; i<p->num-1; i++){
        p->p[i]=p->p[i+1];
    }
    p->p[--p->num]=NULL;
    return idx;
}

int pwlist_free_i(PWList p, int idx){
    Password pw;

    pw=pwlist_remove_i(p, idx);
    if(pw){
        password_free(pw);
        return 0;
    }
    return -1;
}

int pwlist_free_p(PWList p, Password pw){
    if(pwlist_remove_p(p, pw)>=0){
        password_free(pw);
        return 0;
    }
    return -1;
}

int pwlist_recrypt_passes(PWList p){
    int i;
    
    for(i=0; i<p->num; i++){
        if(password_recrypt_pass(p->p[i])) return -1;
    }
    return 0;
}
